package geniusweb.protocol.session.shaop;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.deadline.DeadlineTime;
import geniusweb.protocol.session.SessionSettings;
import geniusweb.protocol.session.TeamInfo;
import geniusweb.references.Parameters;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithParameters;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;
import tudelft.utilities.logging.ReportToLogger;

public class SHAOPSettingsTest extends GeneralTests<SHAOPSettings> {

	private final ObjectMapper jackson = new ObjectMapper();
	private Parameters params = new Parameters();
	private final String serialized = "{\"SHAOPSettings\":{\"participants\":[{\"TeamInfo\":{\"parties\":[{\"party\":{\"partyref\":\"party1\",\"parameters\":{}},\"profile\":\"profile1\"},{\"party\":{\"partyref\":\"party2\",\"parameters\":{}},\"profile\":\"profile1\"}]}},{\"TeamInfo\":{\"parties\":[{\"party\":{\"partyref\":\"party3\",\"parameters\":{}},\"profile\":\"profile1\"},{\"party\":{\"partyref\":\"party4\",\"parameters\":{}},\"profile\":\"profile1\"}]}}],\"deadline\":{\"DeadlineTime\":{\"durationms\":1000}}}}";
	private SHAOPSettings settings1, settings1a, settings2, settings3;
	private TeamInfo pair1, pair2, pair3;

	@Before
	public void before() throws URISyntaxException {
		ProfileRef profile1 = new ProfileRef("profile1");
		PartyWithParameters party1 = new PartyWithParameters(
				new PartyRef("party1"), params);
		PartyWithParameters party2 = new PartyWithParameters(
				new PartyRef("party2"), params);
		PartyWithParameters party3 = new PartyWithParameters(
				new PartyRef("party3"), params);
		PartyWithParameters party4 = new PartyWithParameters(
				new PartyRef("party4"), params);
		PartyWithParameters party5 = new PartyWithParameters(
				new PartyRef("party5"), params);
		PartyWithParameters party6 = new PartyWithParameters(
				new PartyRef("party6"), params);

		PartyWithProfile partyprof1 = new PartyWithProfile(party1, profile1);
		PartyWithProfile partyprof2 = new PartyWithProfile(party2, profile1);
		PartyWithProfile partyprof3 = new PartyWithProfile(party3, profile1);
		PartyWithProfile partyprof4 = new PartyWithProfile(party4, profile1);
		PartyWithProfile partyprof5 = new PartyWithProfile(party5, profile1);
		PartyWithProfile partyprof6 = new PartyWithProfile(party6, profile1);

		DeadlineTime deadline = new DeadlineTime(1000);
		DeadlineTime deadline2 = new DeadlineTime(2000);
		pair1 = new TeamInfo(Arrays.asList(partyprof1, partyprof2));
		pair2 = new TeamInfo(Arrays.asList(partyprof3, partyprof4));
		pair3 = new TeamInfo(Arrays.asList(partyprof5, partyprof6));

		settings1 = new SHAOPSettings(Arrays.asList(pair1, pair2), deadline);
		settings1a = new SHAOPSettings(Arrays.asList(pair1, pair2), deadline);
		settings2 = new SHAOPSettings(Arrays.asList(pair1, pair3), deadline);
		settings3 = new SHAOPSettings(Arrays.asList(pair1, pair2), deadline2);
	}

	@Override
	public List<List<SHAOPSettings>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(settings1, settings1a),
				Arrays.asList(settings2), Arrays.asList(settings3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"SHAOPSettings.*TeamInfo.*PartyWithProfile.*party1.*party2.*TeamInfo.*party3.*party4.*DeadlineTime.*1000.*",
				"SHAOPSettings.*TeamInfo.*PartyWithProfile.*party1.*party2.*TeamInfo.*party5.*party6.*DeadlineTime.*1000.*",
				"SHAOPSettings.*TeamInfo.*PartyWithProfile.*party1.*party2.*TeamInfo.*party3.*party4.*DeadlineTime.*2000.*");
	}

	@Test
	public void getProtocolTest() {
		assertEquals("SHAOP", settings1.getProtocol(new ReportToLogger("test"))
				.getRef().getURI().toString());
	}

	@Test(expected = IllegalArgumentException.class)
	public void constructorNoDeadlineTest() {
		new SHAOPSettings(Arrays.asList(pair1, pair2), null);
	}

	@Test
	public void testMaxRuntime() {
		assertEquals((Double) 1.000, settings1.getMaxRunTime());
	}

	@Test
	public void testDeserialize() throws IOException {
		SessionSettings obj = jackson.readValue(serialized,
				SessionSettings.class);
		System.out.println(obj);
		assertEquals(settings1, obj);
	}

	@Test
	public void testSerialize()
			throws JsonProcessingException, URISyntaxException {

		String string = jackson.writeValueAsString(settings1);
		System.out.println(string);
		assertEquals(serialized, string);
	}

}
